// (C) Copyright 2002-2005 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

//-----------------------------------------------------------------------------
//----- ADSDatabase.cpp : Implementation of ADSDatabase
//-----------------------------------------------------------------------------
#include "StdAfx.h"
#include "ADSDatabase.h"

//-----------------------------------------------------------------------------
ACRX_CONS_DEFINE_MEMBERS(ADSDatabase, AcDbDatabaseReactor, 1)

//-----------------------------------------------------------------------------
ADSDatabase::ADSDatabase (AcDbDatabase *pDb) : AcDbDatabaseReactor(), mpDatabase(pDb) {
	if ( pDb )
		pDb->addReactor (this) ;
}

//-----------------------------------------------------------------------------
ADSDatabase::~ADSDatabase () {
	Detach () ;
}

//-----------------------------------------------------------------------------
void ADSDatabase::Attach (AcDbDatabase *pDb) {
	Detach () ;
	if ( mpDatabase == NULL ) {
		if ( (mpDatabase =pDb) != NULL )
			pDb->addReactor (this) ;
	}
}

void ADSDatabase::Detach () {
	if ( mpDatabase ) {
		mpDatabase->removeReactor (this) ;
		mpDatabase =NULL ;
	}
}

AcDbDatabase *ADSDatabase::Subject () const {
	return (mpDatabase) ;
}

bool ADSDatabase::IsAttached () const {
	return (mpDatabase != NULL) ;
}


// -----------------------------------------------------------------------------
void ADSDatabase::objectOpenedForModify(const AcDbDatabase * dwg, const AcDbObject * dbObj)
{
	//AcTransaction* pTrans = NULL;
	//pTrans = acdbTransactionManager->startTransaction();

	////for testing i will open modal space
	//AcDbObject* obj = NULL;
	//pTrans->getObject(obj, dbObj->ownerId(), AcDb::kForRead);
	//acdbTransactionManager->abortTransaction();	

	AcDbObject* obj = NULL;
	Acad::ErrorStatus es = acdbOpenObject(obj, dbObj->ownerId(), AcDb::kForRead);
	obj->close();

	AcDbDatabaseReactor::objectOpenedForModify (dwg, dbObj) ;
}
