// (C) Copyright 2002-2005 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

//-----------------------------------------------------------------------------
//----- ADSTransaction.cpp : Implementation of ADSTransaction
//-----------------------------------------------------------------------------
#include "StdAfx.h"
#include "ADSTransaction.h"

//-----------------------------------------------------------------------------
ACRX_CONS_DEFINE_MEMBERS(ADSTransaction, AcTransactionReactor, 1)

//-----------------------------------------------------------------------------
ADSTransaction::ADSTransaction (AcApDocument *pDoc) : AcTransactionReactor(), mpDocument(pDoc) {
	if ( pDoc )
		pDoc->transactionManager ()->addReactor (this) ;
}

//-----------------------------------------------------------------------------
ADSTransaction::~ADSTransaction () {
	Detach () ;
}

//-----------------------------------------------------------------------------
void ADSTransaction::Attach (AcApDocument *pDoc) {
	Detach () ;
	if ( mpDocument == NULL ) {
		if ( (mpDocument =pDoc) != NULL )
			pDoc->transactionManager ()->addReactor (this) ;
	}
}

void ADSTransaction::Detach () {
	if ( mpDocument ) {
		mpDocument->transactionManager ()->removeReactor (this) ;
		mpDocument =NULL ;
	}
}

AcApDocument *ADSTransaction::Subject () const {
	return (mpDocument) ;
}

bool ADSTransaction::IsAttached () const {
	return (mpDocument != NULL) ;
}

// -----------------------------------------------------------------------------
void ADSTransaction::transactionEnded(int & numTransactions, AcDbTransactionManager * transactionManagerPtr)
{
	AcTransaction* pTrans = NULL;
	pTrans = acdbTransactionManager->startTransaction();

	//add a new line
	AcDbBlockTable *pBlockTable = NULL;
	acdbTransactionManager->getObject((AcDbObject*&)pBlockTable, acdbHostApplicationServices()->workingDatabase()->blockTableId(), AcDb::kForRead);
	AcDbObjectId id;
	pBlockTable->getAt(ACDB_MODEL_SPACE, id);
	AcDbBlockTableRecord *pSpaceRecord = NULL;
	acdbTransactionManager->getObject((AcDbObject*&)pSpaceRecord, id, AcDb::kForWrite);

	AcDbLine *pLine = new AcDbLine(AcGePoint3d(0,0,0), AcGePoint3d(10,10,0));

	AcDbObjectId objId;
	pSpaceRecord->appendAcDbEntity(objId, pLine);
	acdbTransactionManager->addNewlyCreatedDBRObject(pLine);

	acdbTransactionManager->endTransaction();	

	AcTransactionReactor::transactionEnded (numTransactions, transactionManagerPtr) ;

	
}
