// (C) Copyright 2002-2005 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

//-----------------------------------------------------------------------------
//----- acrxEntryPoint.h
//-----------------------------------------------------------------------------
#include "StdAfx.h"
#include "resource.h"
#include "ADSEditor.h"
#include "TestDlg.h"

//-----------------------------------------------------------------------------
#define szRDS _RXST("ADS")
ADSEditor *m_pEditor = NULL;
//-----------------------------------------------------------------------------


UINT ThreadFunction(LPVOID para)
{
	AcDbBlockTable *pBlockTable = NULL;
	AcDbBlockTableRecord *pSpaceRecord = NULL;

	//Open the block table for read
	Acad::ErrorStatus es = acdbHostApplicationServices()->workingDatabase()->getSymbolTable(pBlockTable, AcDb::kForRead);

	//Open the modal space for write
	es = pBlockTable->getAt(ACDB_MODEL_SPACE, pSpaceRecord, AcDb::kForWrite);

	//close the block table
	es = pBlockTable->close();

	AcGePoint3d pt1, pt2;
	int res = acedGetPoint(NULL, _T("\nSpecify first point:\n"),asDblArray(pt1));
	if(res==RTCAN)
		return 0;

	res = acedGetCorner(asDblArray(pt1), _T("\nSpecify second point:\n"),asDblArray(pt2));
	if(res==RTCAN)
		return 0;

	AcDbLine *pLine = new AcDbLine(pt1, pt2);

	AcDbObjectId objId;
	es = pSpaceRecord->appendAcDbEntity(objId, pLine);
	
	//Not closing newly created Line after adding the line to database
	es = pLine->close();

	//Any one of below can crash the AutoCAD.
	//Not closing modal space which is open for write
	es = pSpaceRecord->close();
	return 0; 
}

void addToModelSpace(AcDbObjectId &objId, AcDbEntity* pEntity)
{
    AcDbBlockTable *pBlockTable;
    AcDbBlockTableRecord *pSpaceRecord;
    acdbHostApplicationServices()->workingDatabase()
        ->getSymbolTable(pBlockTable, AcDb::kForRead);
    pBlockTable->getAt(ACDB_MODEL_SPACE, pSpaceRecord,
        AcDb::kForWrite);
    pBlockTable->close();
    pSpaceRecord->appendAcDbEntity(objId, pEntity);
    pSpaceRecord->close();
    pEntity->close();
    return;
}

//----- ObjectARX EntryPoint
class CTenWaysApp : public AcRxArxApp {

public:
	CTenWaysApp () : AcRxArxApp () {}

	virtual AcRx::AppRetCode On_kInitAppMsg (void *pkt) {
		// TODO: Load dependencies here

		acrxLoadModule(_T("ADSCustomEnt.dbx"),true,true);

		// You *must* call On_kInitAppMsg here
		AcRx::AppRetCode retCode =AcRxArxApp::On_kInitAppMsg (pkt) ;
		
		// TODO: Add your initialization code here
		

		return (retCode) ;
	}

	virtual AcRx::AppRetCode On_kUnloadAppMsg (void *pkt) {
		// TODO: Add your code here

		/*if(m_pEditor != NULL)
		{
			delete m_pEditor;
			m_pEditor = NULL;
		}*/
		// You *must* call On_kUnloadAppMsg here
		AcRx::AppRetCode retCode =AcRxArxApp::On_kUnloadAppMsg (pkt) ;


		return (retCode) ;
	}

	virtual void RegisterServerComponents () {
	}

public:

	//For slide 2: Remember to call AcDbObject::Close()
	//Create a new line between 0,0,0 to 10,10,0
	//Forgets to call close on newly added entity
	static void ADSTenWaysBad1(void)
	{
		AcDbBlockTable *pBlockTable = NULL;
		AcDbBlockTableRecord *pSpaceRecord = NULL;

		//Open the block table for read
		acdbHostApplicationServices()->workingDatabase()->getSymbolTable(pBlockTable, AcDb::kForRead);

		//Open the modal space for write
		pBlockTable->getAt(ACDB_MODEL_SPACE, pSpaceRecord, AcDb::kForWrite);

		//close the block table
		pBlockTable->close();

		//create new Line between 0,0,0 to 10,10,0
		AcDbLine *pLine = new AcDbLine(AcGePoint3d(0,0,0), AcGePoint3d(10,10,0));
	
		AcDbObjectId objId;
		pSpaceRecord->appendAcDbEntity(objId, pLine);
		
		//Not closing newly created Line after adding the line to database
		pLine->close();

		//AutoCAD crashes here.
		//Not closing modal space which is open for write
		pSpaceRecord->close();
		
	}

	//Creates a new line and adds to the model space of the database
	//Closes the newly created entity, but forgets the call close on model space 
	static void ADSTenWaysBad1a(void)
	{
		// Add your code for command ADSTenWays.Way1a here
		AcDbBlockTable *pBlockTable = NULL;
		AcDbBlockTableRecord *pSpaceRecord = NULL;

		//Open the block table for read
		acdbHostApplicationServices()->workingDatabase()->getSymbolTable(pBlockTable, AcDb::kForRead);

		//Open the modal space for write
		pBlockTable->getAt(ACDB_MODEL_SPACE, pSpaceRecord, AcDb::kForWrite);

		//close the block table
		pBlockTable->close();

		//create new Line between 0,0,0 to 10,10,0
		AcDbLine *pLine = new AcDbLine(AcGePoint3d(0,0,0), AcGePoint3d(10,10,0));
	
		AcDbObjectId objId;
		pSpaceRecord->appendAcDbEntity(objId, pLine);
		
		//Any one of below can crash the AutoCAD.
		//Not closing modal space which is open for write
		pSpaceRecord->close();

		//Not closing newly created Line after adding the line to database
		pLine->close();

	}

	//Draw a circle
	//Issue command Bad2
	//Command ask the user to select a entity
	//Select the circle draw in above step
	//Commands deletes the selected object
	static void ADSTenWaysBad2(void)
	{
		int nReturn ;
		ads_name name;
		ads_point pt;

		nReturn = acedEntSel(_T("Select object"), name, pt);

		if(nReturn != RTNORM)
			return;

		//get the object Id of the entity
		AcDbObjectId Id;
		if(acdbGetObjectId(Id, name) != Acad::eOk)
			return;

		//open the selected entity
		AcDbEntity *pEntity = NULL;
		if(acdbOpenAcDbEntity(pEntity, Id, AcDb::kForRead) != Acad::eOk)
		{
			return;
		}

		//do some custom logic and try to delete
		//.....
		//.....
		//.....
		//.....
		//.....
		//.....
		//.....
		//delete pEntity;
		pEntity->close();
		pEntity = NULL;
	}


	//Draw a circle
	//Issue command Bad3
	//Command ask the user to select a entity
	//Select the circle draw in above step
	//Commands changes the radius and closes the selected circle by calling Close()
	//Then once again command try to change the color of the selected object using closed object
	static void ADSTenWaysBad3(void)
	{
		// Add your code for command ADSTenWays.Bad3 here
		int nReturn ;
		ads_name name;
		ads_point pt;

		nReturn = acedEntSel(_T("Select a circle"), name, pt);

		if(nReturn != RTNORM)
			return;

		//get the object Id of the entity
		AcDbObjectId Id;
		if(acdbGetObjectId(Id, name) != Acad::eOk)
			return;

		//open the selected entity
		AcDbEntity *pEntity = NULL;
		
		if(acdbOpenAcDbEntity(pEntity, Id, AcDb::kForWrite) != Acad::eOk)
		{
			return;
		}
		
		AcDbCircle *pCircle = AcDbCircle::cast(pEntity);
		if(pCircle == NULL)
		{
			pEntity->close();
			return;
		}

		pCircle->setRadius(pCircle->radius() * 1.5);

		//Close
		pCircle->close();

		//..............



		//Try to set color as Red
		pCircle->setColorIndex(1);

	}


	static void ADSTenWaysBad3a(void)
	{
		// Add your code for command ADSTenWays.Bad3a here
		int nReturn ;
		ads_name name;
		ads_point pt;

		nReturn = acedEntSel(_T("Select a circle"), name, pt);

		if(nReturn != RTNORM)
			return;

		//get the object Id of the entity
		AcDbObjectId Id;
		if(acdbGetObjectId(Id, name) != Acad::eOk)
			return;

		//open the selected entity
		AcDbEntity *pEntity = NULL;
		
		if(acdbOpenAcDbEntity(pEntity, Id, AcDb::kForRead) != Acad::eOk)
		{
			return;
		}
		
		AcDbCircle *pCircle = AcDbCircle::cast(pEntity);
		if(pCircle == NULL)
		{
			pEntity->close();
			return;
		}

		//Try to modify using read only circle
		pCircle->setRadius(pCircle->radius() * 1.5);

		//Close
		pCircle->close();
	}

	//1.	Command creates a new database
	//2.	Stores the current working database 
	//3.	Uses the function AcDbHostApplicationServices::setWorkingDatabase() to set the 
	//		newly created database as working database
	//4.	Deletes the working database stored in step2 thinking that there is valid working database.
	static void ADSTenWaysBad4(void)
	{
		// Add your code for command ADSTenWays.Bad8 here
		AcDbDatabase *pDatabase = acdbHostApplicationServices()->workingDatabase();
		AcDbDatabase *pNewDatabase = new AcDbDatabase();

		acdbHostApplicationServices()->setWorkingDatabase(pNewDatabase);
		
		//..............some logic


		//............
		delete pDatabase;
		//correct way
		//acdbHostApplicationServices()->setWorkingDatabase(pDatabase);
		//delete pNewDatabase;
	}

	//1.	Issue command Bad5
	//2.	Command tries to create a new line and tries to add to model space
	//3.	As Command is running in application context, it is not possible to open 
	//		the model space for write, opening will return a error code
	//4.	If the command proceeds without checking the error code AutoCAD crashes
	static void ADSTenWaysBad5(void)
	{
		AcDbBlockTable *pBlockTable = NULL;
		AcDbBlockTableRecord *pSpaceRecord = NULL;
		Acad::ErrorStatus es;

		es = acDocManager->lockDocument(acDocManager->curDocument());
		es = acdbHostApplicationServices()->workingDatabase()->getSymbolTable(pBlockTable, AcDb::kForRead);

		if(es != Acad::eOk)
			return;

		//Open the modal space for write
		es = pBlockTable->getAt(ACDB_MODEL_SPACE, pSpaceRecord, AcDb::kForWrite);

		if(es != Acad::eOk)
		{
			pBlockTable->close();
			return;
		}
		//close the block table
		es = pBlockTable->close();

		//create new Line between 0,0,0 to 10,10,0
		AcDbLine *pLine = new AcDbLine(AcGePoint3d(0,0,0), AcGePoint3d(10,10,0));
	
		AcDbObjectId objId;
		es = pSpaceRecord->appendAcDbEntity(objId, pLine);
		
		if(es != Acad::eOk)
		{
			delete pLine;
			pSpaceRecord->close();
			return;
		}
		//Not closing newly created Line after adding the line to database
		pLine->close();

		pSpaceRecord->close();

		acDocManager->unlockDocument(acDocManager->curDocument());
	}

	//1.	Draw a circle
	//2.	Select the circle
	//3.	Issue command Bad6a
	//4.	Command Bad6a prints the dxf name of the selected entity in the command line.
	//5.	Issue command  Bad6
	//6.	Bad6 command loops 129 times reselecting the circle without releasing the 
	//		selection set
	//7.	Know issue command Bad6a, which fails to select. From this point on words
	//		all call to acedSSAdd and acedSSGet will fail.
	static void ADSTenWaysBad6(void)
	{
		int nIndex = 0;
		while(nIndex <= 129) //128 is Maximum limit 
		{
			ads_name ss;
			acedSSGet(L"I", NULL,NULL,NULL,ss); 

			long ssLength = 0;
			acedSSLength(ss, &ssLength);

			if (ssLength == 0)
			{
				acutPrintf(L"\nNothing selected.");
				break;
			}

			ads_name ename;
			AcDbObjectId objId;
			AcDbEntity *pEnt;

			for (long j = 0; j < ssLength; ++j)
			{
				if (RTNORM != acedSSName(ss, j, ename))
				  continue;

				if (Acad::eOk != acdbGetObjectId(objId, ename))
				  continue;

				if (Acad::eOk != acdbOpenAcDbEntity(pEnt, objId, AcDb::kForRead))
				  continue;

				pEnt->close();
			}

			//acedSSFree(ss);
			nIndex++;
		}
	}

	static void ADSTenWaysBad6a(void)
	{
		ads_name ss;
		acedSSGet(NULL, NULL,NULL,NULL,ss); 

		long ssLength = 0;
		acedSSLength(ss, &ssLength);

		if (ssLength == 0)
		{
			acutPrintf(L"\nNothing selected.");
			return;
		}

		ads_name ename;
		AcDbObjectId objId;
		AcDbEntity *pEnt;

		for (long j = 0; j < ssLength; ++j)
		{
			if (RTNORM != acedSSName(ss, j, ename))
			  continue;

			if (Acad::eOk != acdbGetObjectId(objId, ename))
			  continue;

			if (Acad::eOk != acdbOpenAcDbEntity(pEnt, objId, AcDb::kForRead))
			  continue;

			
			::acutPrintf(pEnt->isA()->dxfName());
			acutPrintf(L"\n");
			pEnt->close();
		}

		acedSSFree(ss);
	}

	//1.Issue command Bad7
	//2.This command adds a editor reactor
	//3.Start a line command and create line specifying the start and end point
	//4.Obserb the command line, editor reactors prints the command name which is 
	//	just ended
	//5.Unload the ARX, here TenWay arx is not removing the editor reactor added. 
	//	Due to this AutoCAD crashes
	static void ADSTenWaysBad7(void)
	{
		m_pEditor = new ADSEditor();
	}

	//1.	Draw a circle
	//2.	Issue command Bad8
	//3.	Command asks the user to select a circle,  select circle created in step1
	//4.	Command tries to change the radius of the selected circle, 
	//5.	As the command end the transaction, a new transition is started to add a line. 
	//		In real world a this type of cases comes when user wants to link two entities
	//6.	This will trigger a Recursive Transactions and crashes the AutoCAD.

	static void ADSTenWaysBad8(void)
	{

		int nReturn ;
		ads_name name;
		ads_point pt;

		nReturn = acedEntSel(_T("Select a circle"), name, pt);

		if(nReturn != RTNORM)
			return;

		//get the object Id of the entity
		AcDbObjectId Id;
		if(acdbGetObjectId(Id, name) != Acad::eOk)
			return;

		//open the selected entity
		AcDbEntity *pEntity = NULL;

		AcTransaction* pTrans = NULL;
		pTrans = acdbTransactionManager->startTransaction();

		acdbTransactionManager->getObject((AcDbObject*&)pEntity, Id, AcDb::kForWrite);
		
		
		AcDbCircle *pCircle = AcDbCircle::cast(pEntity);
		if(pCircle == NULL)
		{
			acdbTransactionManager->abortTransaction();
			return;
		}

		//Try to modify using read only circle
		pCircle->setRadius(pCircle->radius() * 1.5);

		//Close
		acdbTransactionManager->endTransaction();
	}

	//1.	Issue Command Bad9
	//2.	command shows a dialog box
	//3.	Press ok on the dialog box
	//4.	Draw a circle, Issue command array and create array of the circle
	//5.	Issue command Bad9. AutoCAD crashes. 
	static void ADSTenWaysBad9(void)
	{
		CAcModuleResourceOverride myResources;

		CTestDlg dlg;
		dlg.DoModal();
	}

	//1.	Issue command Bad 10
	//2.	Command starts a new thread
	//3.	Thread function just tries to add a new line 
	//		to working database by taking two points from use.
	static void ADSTenWaysBad10(void)
	{
		AfxBeginThread(ThreadFunction, NULL);
	}


public:
	// -----------------------------------------------------------------------------
	virtual AcRx::AppRetCode On_kLoadDwgMsg(void * pkt);
public:


public:

	// - ADSTenWays.Bad11 command (do not rename)
	static void ADSTenWaysBad11(void)
	{
		// Add your code for command ADSTenWays.Bad11 here
		ads_name entres;
		ads_point ptres;
		acedEntSel(L"\nSelect Entity to add XData",entres,ptres);

		AcDbObjectId objId = AcDbObjectId::kNull;

		acdbGetObjectId(objId,entres);

		AcDbEntityPointer pEnt(objId,kForRead);

		if(Acad::eOk == pEnt.openStatus())
		{
			// Get the application name and string to be added to
			// xdata.
			//
			TCHAR appName[132]={L"ADSKCrashDemo"}, resString[200];
			resString[0] = _T('\0');

			acedGetString(NULL, _T("\nEnter string to be added: "),
				resString);


			struct  resbuf  *pRb, *pTemp;

			pRb = pEnt->xData(appName);

			if (pRb != NULL) {

				// If xdata is present, then walk to the
				// end of the list.
				//
				for (pTemp = pRb; pTemp->rbnext != NULL;
					pTemp = pTemp->rbnext)
				{ ; }
			} else {
				// If xdata is not present, register the application
				// and add appName to the first resbuf in the list.
				// Notice that there is no -3 group as there is in
				// AutoLISP. This is ONLY the xdata so
				// the -3 xdata-start marker isn't needed.
				// 
				acdbRegApp(appName);

				pRb = acutNewRb(AcDb::kDxfRegAppName);
				pTemp = pRb;
				pTemp->resval.rstring
					= (TCHAR*) malloc((_tcslen(appName) + 1) * sizeof(TCHAR));
				_tcscpy(pTemp->resval.rstring, appName);
			}

			// Add user-specified string to the xdata.
			//
			pTemp->rbnext = acutNewRb(AcDb::kDxfXdAsciiString);
			pTemp = pTemp->rbnext;
			pTemp->resval.rstring
				= (TCHAR*) malloc((_tcslen(resString) + 1) * sizeof(TCHAR));
			_tcscpy(pTemp->resval.rstring, resString);

			// The following code shows the use of upgradeOpen()
			// to change the entity from read to write.
			//
			pEnt->upgradeOpen();
			pEnt->setXData(pRb);
			acutRelRb(pRb);
		}
	}
public:

	// - ADSTenWays._smrtptrs command (do not rename)
	static void ADSTenWays_smrtptrs(void)
	{
		// Add your code for command ADSTenWays._smrtptrs here

		AcDbObjectPointer<AcDbLine> pLine;

		pLine.create();
		pLine->setDatabaseDefaults();
		pLine->setStartPoint(AcGePoint3d(10,10,0));
		pLine->setEndPoint(AcGePoint3d(20,20,0));

		AcDbDatabase *pDb = acdbHostApplicationServices()->workingDatabase();

		AcDbBlockTableRecordPointer pMdlSpc(ACDB_MODEL_SPACE,pDb,AcDb::kForWrite);

		if(Acad::eOk == pMdlSpc.openStatus())
		{
			pMdlSpc->appendAcDbEntity(pLine);
		}
	}
} ;

//-----------------------------------------------------------------------------
IMPLEMENT_ARX_ENTRYPOINT(CTenWaysApp)

ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad1, Bad1, ACRX_CMD_SESSION, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad1a, Bad1a, ACRX_CMD_TRANSPARENT, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad2, Bad2, ACRX_CMD_TRANSPARENT, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad3, Bad3, ACRX_CMD_TRANSPARENT, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad3a, Bad3a, ACRX_CMD_TRANSPARENT, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad4, Bad4, ACRX_CMD_TRANSPARENT, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad5, Bad5, ACRX_CMD_SESSION, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad6, Bad6, ACRX_CMD_USEPICKSET | ACRX_CMD_REDRAW , NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad6a, Bad6a, ACRX_CMD_TRANSPARENT, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad7, Bad7, ACRX_CMD_TRANSPARENT, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad8, Bad8, ACRX_CMD_TRANSPARENT, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad9, Bad9, ACRX_CMD_SESSION, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad10, Bad10, ACRX_CMD_TRANSPARENT, NULL)
// -----------------------------------------------------------------------------

AcRx::AppRetCode CTenWaysApp::On_kLoadDwgMsg(void * pkt)
{
	AcRx::AppRetCode retCode =AcRxArxApp::On_kLoadDwgMsg (pkt) ;

	DocVars.docData().m_Transaction = new ADSTransaction(acDocManager->curDocument ());

	return (retCode) ;
}
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, Bad11, Bad11, ACRX_CMD_TRANSPARENT, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CTenWaysApp, ADSTenWays, _smrtptrs, smrtptr, ACRX_CMD_TRANSPARENT, NULL)
